#!/usr/bin/env python
# Copyright 2012 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Update subcommand: updates a Launchpad development environment."""

__metaclass__ = type
__all__ = [
    'SubCommand',
    ]

import os.path
from shelltoolbox import (
    cd,
    mkdirs,
    run,
    )

from lpsetup import argparser
from lpsetup import handlers
from lpsetup.settings import (
    LP_CODE_DIR,
    LP_SOURCE_DEPS,
    )


def initialize_directories(working_dir, external_path):
    """Initialize the eggs, yui, and sourcecode directories.

    Create them if necessary.
    """
    for dir_ in ['eggs', 'yui', 'sourcecode']:
        mkdirs(os.path.join(working_dir, external_path, dir_))


def update_dependencies(working_dir, external_path, use_http):
    """Update the external dependencies."""
    use_http_param = '--use-http' if use_http else None
    cmd = os.path.join(working_dir, 'utilities', 'update-sourcecode')
    abs_external_path = os.path.abspath(
        os.path.join(working_dir, external_path))
    source_path = os.path.join(abs_external_path, 'sourcecode')
    run(cmd, use_http_param, source_path)

    # Update the download cache.
    download_cache = os.path.join(working_dir, 'download-cache')
    if os.path.exists(download_cache):
        run('bzr', 'up', download_cache)
    else:
        run('bzr', 'co', '-v', '--lightweight', LP_SOURCE_DEPS, download_cache)

    # Link to the external sourcecode.
    if abs_external_path != working_dir:
        cmd = os.path.join(
            working_dir, 'utilities', 'link-external-sourcecode')
        run(cmd,
            '--target', working_dir,
            '--parent', external_path)


def update_tree(working_dir):
    """Update the tree at working_dir with the latest LP code."""
    with cd(working_dir):
        run('bzr', 'pull')


class SubCommand(argparser.StepsBasedSubCommand):
    """Updates an existing Launchpad development environment.

    Gets new versions of Launchpad source and external sources.
    """

    steps = (
        (initialize_directories, 'working_dir', 'external_path'),
        (update_dependencies, 'working_dir', 'external_path', 'use_http'),
        (update_tree, 'working_dir'),
        )
    help = __doc__
    handlers = (
        # Normalize paths and default to cwd if none exists.
        handlers.handle_working_dir,
        )

    @staticmethod
    def add_common_arguments(parser):
        parser.add_argument(
            '-e', '--external-path', default='.',
            help='Path to directory that contains sourcecode '
                 'and download-cache directories, relative to the '
                 'working-dir. ')

    def add_arguments(self, parser):
        super(SubCommand, self).add_arguments(parser)
        self.add_common_arguments(parser)
        parser.add_argument(
            '--use-http', default=False, action='store_true',
            help='Force bzr to use http to get the sourcecode '
                 'branches rather than using bzr+ssh.')
        parser.add_argument(
            '-W', '--working-dir', default=LP_CODE_DIR,
            help='Path to branch to update.  '
                 '[DEFAULT={0}]'.format(LP_CODE_DIR))
